/**
 * 数组列表区域
 * Copyright(c) Alibaba Group Holding Limited.
 * MIT Licensed
 */

import React from 'react';
import { SortableContainer, arrayMove } from 'react-sortable-hoc';

import { Icon, Alert } from 'antd';
import schemaUtil from 'schema-util';
import assign from 'object-assign';

import ArrayItem from './array_item';
import ExcelButton from './excel_button';
import clone from '../util/clone';

const MAX_ARRAY_ITEM_LENGTH = 500;

class FieldArray extends React.Component {
  // 添加按钮
  displayAddButton() {
    const { schema, data, config } = this.props;
    if (schema.meta && Array.isArray(data)) {
      const arrLength = data.length;
      const maxLength = schema.meta.maxLength ? schema.meta.maxLength : null;
      if (maxLength === arrLength) {
        return (
          <p>{config.locale.ARRAY_MAX_LENGHT_ERROR.replace('%s', maxLength)}</p>
        );
      }
    }
    return (
      <div className="add-btn">
        <a onClick={ this.props.addItem }><Icon type="plus" />{config.locale.ADD_ARRAY_ITEM}</a>
      </div>
    );
  }

  onUploadSuccess = data => {
    this.props.onUploadSuccess({
      value: data,
      fields: [ this.props.field ],
      type: 'array-import',
    });
  }

  onExcelBtnClick = data => {
    if (this.props.onClick) {
      this.props.onClick(data);
    }
  }

  renderItems(isPlainObject) {
    const { actives, data, schema, config } = this.props;
    const json = assign({}, schema, { type: 'object' });

    if (!Array.isArray(data) || !data.length) {
      return null;
    }

    const itemLength = data.length;
    let hasExtras = false;
    let isHideDelete = false; // 是否显示删除按钮
    if (schema.meta && schema.meta.minLength && data.length <= schema.meta.minLength) {
      isHideDelete = true;
    }

    const ret = data.map((item, index) => {
      if (index > MAX_ARRAY_ITEM_LENGTH && isPlainObject) {
        hasExtras = true;
        return null;
      }

      return (
        <ArrayItem
          key={index}
          index={index}
          field={index}
          itemLength={itemLength}
          disabled={actives[index] && isPlainObject}
          isActived={actives[index] && isPlainObject}
          schema={json}
          data={item}
          isHideDelete={isHideDelete}
          onChange={this.props.onChange}
          copyItem={this.props.copyItem(index)}
          deleteItem={this.props.deleteItem(index)}
          toggleItem={this.props.toggleItem}
          collapseItems={this.props.collapseItems}
          config={config}
        />
      );
    });

    if (hasExtras) {
      ret.push(<div className="row row-extras" key="extra">
        <Alert message={config.locale.ARRAY_TOO_LONG_WARN} type="warning" showIcon /></div>);
    }

    return ret;
  }

  render() {
    const { schema, data, config, field, isNotPlainObject } = this.props;
    const isPlainObject = !isNotPlainObject;

    const excelBtn = config.excelImport && isPlainObject ?
      <ExcelButton
        data={ data }
        schema={ schema }
        title={ schema.description }
        errorList={ this.props.errorList }
        onUploadSuccess={ this.onUploadSuccess }
        onClick={ this.onExcelBtnClick }
        locale={ config.locale }
      />
      : null;
    const fieldCls = field ? 'has-field' : '';

    return (
      <div className={`ui-form-item field-array ${fieldCls}`}>
        <div className="ui-label">
          <span
            className="ui-label-title"
            title={ field && schema.description || null }
          >{ field && schema.description || null }</span>
          {excelBtn}
        </div>
        <div className="ui-content" key={field}>
          <div className="item-array">
            {this.renderItems(isPlainObject)}
          </div>
          { this.displayAddButton() }
        </div>
      </div>
    );
  }
}

const Sortable = SortableContainer(props => {
  return (
    <FieldArray
      { ...props }
    />
  );
});

export class SortArray extends React.Component {
  constructor(props) {
    super(props);

    this.isNotPlainObject = Object.keys(props.schema.properties).some(o => {
      return [ 'array', 'object' ].indexOf(props.schema.properties[o].type) > -1;
    });

    this.state = { actives: {} };
  }

  // 生成随机 uid
  genUID() {
    return (Date.now() + Math.random().toString().substr(2, 3))
      .replace(/^(\d{4})(\d{4})(\d{4})(\d{4})/g, '$1-$2-$3-$4');
  }

  /*
   * @param {Object} properties schema 的 properties
   * @param {Object} 需要更新 uuid 的数据
   * @return {Object} 更新后的数据
   * 复制一个 array 项时， 针对 box 类型的特殊处理
   * - 删除 uuid, uuid应该是自动生成的
   * - left, top 位置偏移，避免多个重叠在一起
   */
  resetBoxData(properties, itemData) {
    const newItemData = clone(itemData);

    Object.keys(properties).forEach(key => {
      if (properties[key].type === 'box') {
        newItemData[key].uid = this.genUID();
        // 增加偏移
        newItemData[key].left += 10;
        newItemData[key].top += 10;
      }
    });

    return newItemData;
  }

  // 收起所有条目
  handleCollapseItems() {
    this.setState({
      actives: {},
    });
  }

  // 展开&收起一个条目，展开时其他条目收起
  handleToggleItem = index => {
    return ev => {
      ev.stopPropagation();
      ev.preventDefault();

      // 直接代开二级面板
      if (this.isNotPlainObject) {
        // 激活二级
        return this.onChange({ selected: true, fields: [ index ] });
      }

      const actives = this.state.actives;
      this.setState({
        actives: { [index]: !actives[index] },
      });
    };
  }

  // 删除条目
  handleDeleteItem = index => {
    return ev => {
      const field = this.props.field;
      const data = this.props.data.slice();
      data.splice(index, 1);

      this.props.onChange({
        value: data,
        fields: [ field ],
        type: 'array-delete',
        schema: this.props.schema,
      });

      ev.stopPropagation();
      ev.preventDefault();
    };
  }

  getNewItem() {
    const data = this.props.data;
    if (data && data.length) {
      return assign(clone(data[0]), { __key: undefined });
    }

    const mockData = schemaUtil.mock(this.props.schema);
    mockData[0].__key = undefined;
    return mockData[0];
  }

  // 新增一个条目
  handleAddItem = () => {
    const { field, onChange, data, schema } = this.props;
    const len = data ? data.length : 0;
    const self = this;
    // clone data
    // default empty array
    const newData = data ? data.slice() : [];
    newData.push(this.resetBoxData(schema.properties, this.getNewItem()));

    onChange({
      value: newData,
      fields: [ field ],
      type: 'array-add',
      schema,
    });

    setTimeout(() => {
      self.setState({
        actives: { [len]: true },
      });
    }, 100);
  }

  // 修改内容
  handleChange = e => {
    const field = this.props.field;
    if (field !== undefined) {
      e.fields.unshift(field);
    }

    this.props.onChange(e);
  }

  // 组件排序
  handleSortItem = ({ oldIndex, newIndex }) => {
    // clone
    let data = this.props.data.slice();
    data = arrayMove(data, oldIndex, newIndex);

    this.props.onChange({
      value: data,
      fields: [ this.props.field ],
      type: 'array-sort',
    });
  }

  // 复制数组项
  handleCopyItem = index => {
    const { field, onChange, data, schema } = this.props;
    const len = index + 1;

    return ev => {
      const originData = data.slice();
      const newData = originData[index];

      if (!newData) { return; }

      originData.splice(index, 0, this.resetBoxData(schema.properties, newData));

      onChange({
        value: originData,
        fields: [ field ],
        type: 'array-add',
        schema,
      });

      setTimeout(() => {
        this.setState({
          actives: { [len]: true },
        });
      }, 100);

      ev.stopPropagation();
      ev.preventDefault();
    };
  }

  // excel 上传成功
  handleUploadSuccess = e => {
    this.props.onChange(e);
  }

  onChange(e) {
    const field = this.props.field;

    if (field !== undefined) {
      e.fields.unshift(field);
    }

    this.props.onChange(e);
  }

  render() {
    const props = this.props;

    return (
      <Sortable
        axis="y"
        lockAxis="y"
        helperClass="schema-editor-array-item-dragging"
        onSortEnd={ this.handleSortItem }
        useDragHandle
        {...props}
        actives={ this.state.actives }
        onUploadSuccess={ this.handleUploadSuccess }
        onChange={ this.handleChange }
        addItem= { this.handleAddItem }
        isNotPlainObject={ this.isNotPlainObject }
        copyItem={ this.handleCopyItem }
        deleteItem={ this.handleDeleteItem }
        toggleItem={ this.handleToggleItem }
        collapseItems={ this.handleCollapseItems }
      />
    );
  }

}

export default SortArray;
