import React from 'react';
import moment from 'moment';
import { Icon, Spin, Modal, Tooltip } from 'antd';

import csvUtil from '../util/csv';
const csv = require('schema-util').csv;

const UPLOAD_STATUS = {
  loading: '正在导入 excel ...',
  error: <span>上传失败</span>,
  success: '上传成功',
  warning: <span>上传成功，但内容有错误</span>,
};
// 找出 date 类型的 schema key
const getDateKey = schema => {
  return Object.keys(schema.properties).filter(key => schema.properties[key].type === 'date');
};

// excel上传&下载组件，上传完成后提供onSuccess回调方法
class ExcelButton extends React.Component {
  constructor() {
    super();
    this.state = {
      uploading: false,
      status: 'loading',
      message: '',
      bigFile: false,
    };
  }

  handleCloseModal = () => {
    this.setState({ uploading: false });
  }

  handleUploadCsv = e => {
    if (this.props.onClick) {
      this.props.onClick({ key: 'handleUploadCsv' });
    }
    const input = e.target;
    const file = input.files.item(0);
    const schema = this.props.schema;
    const dateKeys = getDateKey(schema);
    // 大于 100k 才有 loading 浮层
    this.setState({
      uploading: true,
      bigFile: file.size > 100000,
      status: 'loading',
      message: '',
    });

    if (file.size > 1024 * 1024 * 3) {
      this.setState({
        status: 'error',
        message: '无法上传超过 3M 的文件',
      });
      return;
    }

    setTimeout(() => {
      csvUtil
        .read(file, this.props.title)
        .then(csvData => {
          const data = csv.read(schema, csvData);
          if (dateKeys.length) {
            data.map(dataItem => {
              dateKeys.forEach(dateKey => {
                const m = moment(dataItem[dateKey]);
                if (m.isValid()) {
                  // YYYY-MM-DD HH:mm 式数据转时间戳
                  dataItem[dateKey] = Number(m.format('x'));
                } else {
                  // 兼容科学计数转时间戳
                  const timestamp = Number(dataItem[dateKey]);
                  if (!isNaN(timestamp)) dataItem[dateKey] = timestamp;
                }
              });
              return dataItem;
            });
          }
          this.props.onUploadSuccess(data);
          return this.props.errorList && this.props.errorList.length > 0;
        })
        .then(isHandleError => {
          // 处理错误
          if (isHandleError) {
            this.setState({
              status: 'warning',
              message: this.props.errorList.map(error => (
                <div>第{ error.index + 1 }行的{ error.label }字段不符合规则：{ error.errorMsg }</div>
              )),
            });

            return;
          }

          // 成功
          this.setState({
            status: 'success',
            message: '',
          });

          setTimeout(() => {
            this.setState({
              status: null,
              message: '',
            });
          }, 1000);
        })
        .catch(err => {
          this.setState({
            status: 'error',
            message: '上传的文件内容不合法',
          });
          throw err;
        });
      input.value = '';
    });
  }

  handleDownloadCsv = () => {
    if (this.props.onClick) {
      this.props.onClick({ key: 'handleDownloadCsv' });
    }
    const { title, schema, data } = this.props;
    // 格式化 date 数据
    if (schema.type === 'array') {
      const dateKeys = getDateKey(schema);
      if (dateKeys.length) {
        data.map(dataItem => {
          dateKeys.forEach(dateKey => {
            const m = moment(dataItem[dateKey]);
            if (m.isValid()) {
              dataItem[dateKey] = m.format('YYYY-MM-DD HH:mm');
            }
          });
          return dataItem;
        });
      }
    }
    csvUtil.write(schema, data, title);
  }

  render() {
    if (this.props.rule) {
      return null;
    }
    const locale = this.props.locale;

    let model;
    const uploading = this.state.uploading;
    const bigFile = this.state.bigFile;
    if (bigFile && uploading) {
      // 如果是大文件，才显示 loading 浮层
      model = (<Modal
        visible={uploading}
        onCancel={this.handleCloseModal}
        footer={null}
      >
        <div className="prerelease-box release-box">
          <div className="title">
            {UPLOAD_STATUS[this.state.status]}
          </div>
          <div className="content">
            {this.state.status === 'loading' ? (<Spin />) : null}
            {this.state.status === 'error' && this.state.message }
          </div>
        </div>
      </Modal>);
    }

    return (
      <ul className="item-csvdata">
        <li onClick={this.handleDownloadCsv} >
          <Icon type="download" />{locale.EXPORT_EXCEL}
        </li>
        <li className="item-csvdata-upload">
          <Tooltip visible={ [ 'error', 'warning' ].indexOf(this.state.status) > -1 } title={UPLOAD_STATUS[this.state.status]}>
            <Icon type="upload" />
            {locale.IMPORT_EXCEL}
            <input
              type="file"
              onChange={this.handleUploadCsv}
              accept="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
            />
          </Tooltip>
        </li>
        {model}
      </ul>
    );
  }
}


module.exports = ExcelButton;
