import React from 'react';
import { Icon } from 'antd';
import schemaUtil from 'schema-util';
import addKey from '../util/addKey';
import evaluate from '../util/evaluate';

import Item from './item';

module.exports = class FieldObject extends React.Component {

  onChange = e => {
    const field = this.props.field;
    if (field !== undefined) {
      e.fields.unshift(field);
    }

    this.props.onChange(e);
  }

  onSelectItem = field => {
    return () => {
      this.onChange({ selected: true, fields: [ field ] });
    };
  }

  // 嵌套时获取字段的标题
  getFieldTitle() {
    const { field, schema } = this.props;
    if (field && typeof field !== 'number') {
      return <div className="ui-title">{schema.description}</div>;
    }
    return null;
  }

  isPlainObject() {
    const { schema } = this.props;
    const total = { array: 0, object: 0 };

    Object.keys(schema.properties).forEach(o => {
      const type = schema.properties[o].type;
      if (type === 'array') {
        total.array += 1;
      }

      if (type === 'object') {
        total.object += 1;
      }
    });
    // 一个数组的情况，不折叠
    // 存在对象、或者多个数组的情况折叠
    return total.object > 0 || total.array > 1;
  }

  getItem(key) {
    const { schema, data, config, getCustomField } = this.props;
    const props = schema.properties[key];
    const type = props.type;

    const isPlainObject = this.isPlainObject();

    let itemVal;
    if (typeof data === 'undefined') {
      itemVal = addKey(schemaUtil.mock(props));
    } else {
      itemVal = data[key];
    }

    const itemProps = {
      field: key,
      data: itemVal,
      schema: props,
      parentData: data,
      onChange: this.onChange,
      onClick: this.props.onClick,
      config,
    };

    const ifState = props.meta ? props.meta.if : null;

    if (ifState) {
      try {
        const shouldShow = evaluate(data, ifState);
        if (!shouldShow) {
          return null;
        }
      } catch (e) {
        // 忽略错误
      }
    }

    // 自定义字段，基于 schema 判断逻辑
    if (getCustomField) {
      const customField = getCustomField(itemProps, schema);
      // 不等于 false 的情况
      if (customField !== false) {
        return customField;
      }
    }

    if (isPlainObject && (type === 'object' || type === 'array')) {
      return (
        <div className="ui-object-folder" key={key} onClick={this.onSelectItem(key)}>
          <div>{props.description}</div>
          <span className="ui-object-folder-editor" name={key} title={config.locale.EDIT_OBJECT}>
            <Icon type="edit" />
          </span>
        </div>
      );
    }

    const item = Item.getItem(type, itemProps);
    return <div key={key} className={`field-item field-${type}`}>{item}</div>;
  }

  renderGroups() {
    const { schema, getGroupsRule } = this.props;
    const properties = schema.properties;
    const isPlainObject = this.isPlainObject();
    // 用来分组 object、array
    let objectGroup = [];

    let items = Object.keys(properties).map((field, index) => {
      const type = properties[field].type;
      const itemField = this.getItem(field);

      if (isPlainObject && (type === 'object' || type === 'array')) {
        // 当某个字段类型为 object|array 时，把此字段放入 group，返回 null
        objectGroup.push(itemField);
        return null;
      }

      // 当下一个字段类型不是 object|array 时，如果 objectGroup 为空则返回当前的 field
      if (objectGroup.length === 0) {
        return itemField;
      }

      // 当下一个字段类型不是 object|array 时，如果 objectGroup 非空则 objectGroup + field
      const data = (<div key={`object-group-${index}`} className="ui-object-group">{ objectGroup.map(d => d) }</div>);
      // 清空当前 group
      objectGroup = [];

      return ([ data, itemField ]);
    });

    // 所有的字段都遍历完成后，再追加 objectGroup 还有数据（如果有的话）
    if (objectGroup.length) {
      items = items.concat(<div key={`object-group-${Date.now()}`} className="ui-object-group">{ objectGroup }</div>);
      objectGroup = [];
    }

    if (getGroupsRule) {
      return getGroupsRule(items);
    }

    return items;
  }

  render() {
    return (
      <div className="ui-object">
        {this.getFieldTitle()}
        {this.renderGroups()}
      </div>
    );
  }
};
