import React from 'react';
import classnames from 'classnames';
import { Upload, Input, Icon } from 'antd';

import Copy from '../util/copy';
import UploaderValidator from '../util/uploaderValidator';

/*
 * 上传输入框
 * 属性：
 *  name: 字段名称
 *  label: 中文标题
 *  value: 默认值
 *  meta:
 *    校验参数，Exp: width、maxWidth、height、maxHeight
 *    图片独有参数：setDefaultColorTo 指定图片上传完成后填充图片底色到目标字段
 * 方法：
 *  onChange: 图片上传成功后的回调，返回为数：
 *    target: {value: '图片地址'}
 */
class Uploader extends React.Component {

  constructor() {
    super();
    this.state = { errorMsg: '', success: false };
  }

  // 点击输入框选择文件时隐藏错误提示
  handleSelectFile = () => {
    this.setState({ errorMsg: '' });
  }

  beforeUpload = file => {
    const { meta, config, schema } = this.props;
    const isOverMaxSize = UploaderValidator.validateImageMaxSize(file, schema);
    if (!isOverMaxSize.result) {
      this.setState({ errorMsg: isOverMaxSize.message });
      return false;
    }
    if (Math.floor(file.size) > config.uploadImgSize) {
      this.setState({ errorMsg: `图片大小超过 ${Math.floor(config.uploadImgSize / 1024)}KB` });
      return false;
    }
    if (meta && meta.setDefaultColorTo && file.type.match('image.*')) {
      const reader = new FileReader();
      reader.onloadend = function(evt) {
        if (evt.target.readyState === FileReader.DONE) {
          const img = new Image();
          img.src = evt.target.result;
        }
      };
      reader.readAsDataURL(file);
    }
    return true;
  }

  // 上传的过程
  handleProcess = info => {
    const status = info.file.status;

    switch (status) {
      case 'error': {
        this.setState({
          errorMsg: info.file.response,
        });
        break;
      }
      case 'done': {
        // 清除出错状态
        this.setState({ errorMsg: '', success: true });
        setTimeout(() => {
          this.setState({
            success: false,
          });
        }, 2000);
        const field = this.props.field;
        this.props.onChange({
          value: info.file.response.data.url,
          fields: [ field ],
        });
        break;
      }
      default: {
        break;
      }
    }
  }

  // 直接修改上传地址
  handleChangeImage = e => {
    // 清除出错状态
    this.setState({ errorMsg: '' });
    const { field } = this.props;
    this.props.onChange({
      value: e.target.value,
      fields: [ field ],
    });
  }

  render() {
    // 限制内容提醒
    const { data, field, schema, config } = this.props;
    let metaNote = null;
    const meta = schema.meta;
    const errorMsg = config.validate(data, meta, schema.type) || this.state.errorMsg;
    const cls = classnames({
      'ui-form-item': true,
      'item-image': true,
      'item-image-upload': config.imageUploadOnly,
    });

    if (meta) {
      // 解决冲突
      if (meta.width && meta.maxWidth) {
        delete meta.maxWidth;
      }
      if (meta.height && meta.maxHeight) {
        delete meta.maxHeight;
      }

      // 用校验来获得提示文案
      const validateImageDimension = UploaderValidator.validateImageDimension(meta);
      const validateImageMaxSize = UploaderValidator.validateImageMaxSize({ size: Number.POSITIVE_INFINITY }, { meta });
      if (!validateImageDimension.result || !validateImageMaxSize.result) {
        const both = !validateImageDimension.result && !validateImageMaxSize.result;
        metaNote = <p>图片要求{validateImageDimension.message}{both ? ',' : ''}{validateImageMaxSize.message}</p>;
      }
    }
    let btn = (
      <span onClick={this.handleSelectFile}>
        <Icon type="picture" />{config.locale.CHOOSE_IMAGE}
      </span>
    );
    if (this.state.success) {
      btn = (
        <span className="item-image-success">
          <Icon type="check-circle" />上传成功
        </span>
      );
    }
    const preview = data ? (
      <div className="item-image-preview">
        <div style={{ backgroundImage: `url(${data})` }} />
        <Copy text={data} className="item-image-preview-url" description="复制图片地址">
          <span>{(data || '').split('/').reverse()[0]}</span>
        </Copy>
      </div>
    ) : null;

    // 渲染
    return (
      <div className={cls} key={field}>
        <div className="ui-label">
          <span className="ui-label-title" title={schema.description}>
            {schema.description}
            {
              meta && meta.description
                ? <p title={ schema.meta.description }>{ schema.meta.description }</p>
                : null
            }
            {metaNote}
          </span>
          <Upload
            action={config.uploadApi}
            headers={ config.uploadHeaders || {} }
            beforeUpload={this.beforeUpload}
            onChange={this.handleProcess}
            data={meta}
            withCredentials
            accept="image/*"
            forceAjax
          >
            {btn}
          </Upload>
        </div>
        <div className="ui-content">
          {
            config.imageUploadOnly
              ? null
              : <Input
                size={config.size}
                defaultValue={data}
                value={data}
                onChange={this.handleChangeImage}
              />
          }
          {
            errorMsg && errorMsg !== ''
              ? <div className="item-image-error">
                <Icon type="close-circle" />{errorMsg}
              </div>
              : null
          }
          {
            config.imageUploadOnly
              ? preview
              : null
          }
        </div>
      </div>
    );
  }
}

module.exports = Uploader;
