import React from 'react';
import { Breadcrumb } from 'antd';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import schemaUtil from 'schema-util';

import Item from './fields/item';
import assign from 'object-assign';
import validate from './util/validate';
import clone from './util/clone';
import getArrayTitle from './util/getArrayTitle';
// 对数组增加唯一的 key
import { uid, addDevToolApi, removeDevToolApi } from './util/';

const defaultConfig = {
  excelImport: false, // 开启 excel 导入导出功能
  uploadApi: '/api/upload',
  uploadApiForRichText: '/api/upload?type=richText',
  // custom field type
  fieldMap: {},
  validate,
  uploadImgSize: 512000, // 500KB
  // 只允许上传图片不允许输入 url
  imageUploadOnly: false,
  lang: 'zh_cn',
  size: 'large',
};

class Editor extends React.Component {
  constructor(props) {
    super(props);
    this.state = { data: props.data, selected: [] };
    this.lastItem = null;
    this.__uid = uid(20);

    this.devtoolInit();
  }

  devtoolInit() {
    addDevToolApi(this.__uid, {
      importData: data => {
        if (this.props.onChange) {
          this.props.onChange(data, {});
        }
        if (this.state.data) {
          this.setState({ data });
        }
      },
      exportData: () => this.getValue(),
    });
  }

  componentWillUnmount() {
    removeDevToolApi(this.__uid);
  }

  onItemSelect = e => {
    this.setState({ selected: this.state.selected.concat(e.fields) });
  }

  getValue() {
    const value = this.props.value;
    // 默认去 props.value ，否则去 state.data 内部维护数据
    let data = value || this.state.data;
    if (!data) {
      // if data is null, then mock data first
      data = schemaUtil.mock(this.props.schema);
    }
    return data;
  }

  onChange = e => {
    if (e.selected) {
      this.onItemSelect(e);
      return;
    }

    if (this.props.onBeforeChange) {
      this.props.onBeforeChange(e);
    }

    e.fields = this.state.selected.concat(e.fields).filter(o => o !== undefined);
    // clone data
    let data = JSON.parse(JSON.stringify(this.getValue()));
    const fields = e.fields.slice();
    let tmp = data;
    let tmpSchema = this.props.schema;
    const last = fields.pop();

    // modify array
    if (last === undefined && tmpSchema.type === 'array') {
      data = e.value;
    } else {
      // modify item field
      fields.forEach(item => {
        if (typeof item !== 'number') {
          tmpSchema = tmpSchema.properties[item];
        }

        // if data is null, mock data 或 非数组或对象，都需要 mock
        if (typeof tmp[item] === 'undefined' || typeof tmp[item] !== 'object') {
          tmp[item] = schemaUtil.mock(tmpSchema);
        }

        tmp = tmp[item];
      });
      tmp[last] = e.value;
    }

    if (this.props.onChange) {
      this.props.onChange(data, e);
    }

    this.lastItem = e.fields;
    if (this.state.data) {
      this.setState({ data });
    }
  }

  onClick = data => {
    if (this.props.onClick) {
      this.props.onClick(data);
    }
  }

  validate(fn, locale) {
    return (value, meta, type) => {
      let errorMsg = '';

      switch (type) {
        case 'image':
          if (value && !/^[\w\d\_-]*:\/\/[^\s]*$/gi.test(value)) {
            errorMsg = locale.IMAGE_FORMAT_ERROR;
          }
          break;
        case 'url':
          if (value && /\s/g.test(value)) {
            errorMsg = locale.URL_FORMAT_ERROR;
          }
          break;
        default:
          errorMsg = fn(value, meta, type, locale);
      }

      if (errorMsg && this.props.onError) {
        this.props.onError({
          errorMsg,
          fieldPath: clone(this.lastItem),
        });
      }
      return errorMsg;
    };
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (nextProps.schema !== this.props.schema) {
      this.setState({ data: nextProps.data, selected: [] });
    }
  }

  getDataAndSchema() {
    let schema = this.props.schema;
    let data = this.getValue();
    this.state.selected.forEach(item => {
      if (typeof item !== 'number') {
        schema = schema.properties[item];
      } else {
        // 数组的元素，把类型改为 object 既可
        schema = assign({}, schema, { type: 'object' });
      }
      data = data[item];
    });
    return { data, schema };
  }

  stepBack(index) {
    return () => {
      const selected = this.state.selected.slice();
      while (index !== selected.length) {
        selected.pop();
      }
      this.setState({ selected });
    };
  }

  getNavBar() {
    let { schema, getNavBar, value } = this.props;
    const { selected } = this.state;
    let data = this.getValue();
    const len = selected.length - 1;
    const getNavBarItem = (title, index, needHref) => {
      if (!needHref) {
        return <Breadcrumb.Item key={index}>{title}</Breadcrumb.Item>;
      }
      return <Breadcrumb.Item key={index} onClick={this.stepBack(index)}>{title}</Breadcrumb.Item>;
    };

    const navbarItemList = [ getNavBarItem(schema.description, 0, len > -1, selected[0]) ];
    selected.forEach((key, i) => {
      data = data[key];
      if (typeof key !== 'number') {
        schema = schema.properties[key];
        navbarItemList.push(getNavBarItem(schema.description, i + 1, i < len, selected[i + 1]));
      } else {
        navbarItemList.push(getNavBarItem(getArrayTitle(data, schema, value), i + 1, i < len, selected[i + 1]));
      }
    });

    const nav = <Breadcrumb separator=">">{navbarItemList}</Breadcrumb>;

    if (typeof getNavBar === 'function') {
      return getNavBar(nav);
    }

    return nav;
  }

  render() {
    const { schema, data } = this.getDataAndSchema();
    const config = assign({
      rootData: this.getValue(),
      paths: this.state.selected,
    }, defaultConfig, this.props.config);

    const cls = classnames({
      'schema-editor': true,
      'schema-editor-group': this.state.selected.length > 0,
    });

    const locale = require('./i18n/' + config.lang);
    config.locale = config.locale || locale;
    config.validate = this.validate(config.validate, locale);

    const props = {
      schema,
      data,
      onChange: this.onChange,
      onClick: this.onClick,
      config,
    };

    return (
      <div className={cls} data-schema-uid={this.__uid}>
        <div className="schema-editor-main">
          {this.getNavBar()}
          <div className="schema-editor-scroll">
            <div className="schema-editor-container">
              {Item.getItem(schema.type, props)}
            </div>
          </div>
        </div>
      </div>
    );
  }
}

Editor.propTypes = {
  schema: PropTypes.object,
  config: PropTypes.object,
};

module.exports = Editor;

