/**
 * 上传器校验，单例
 * Copyright(c) Alibaba Group Holding Limited.
 * MIT Licensed
 *
 * Authors:
 *   崖鹰 <zhao.wu@alipay.com> (http://www.neoease.com)
 */


class Validator {

  /**
   * 验证图片尺寸
   * @param  {JSON} requiredDimension 要求尺寸
   * @param  {JSON} realDimension     实际尺寸
   * @return {JSON}                   验证结果和返回信息
   */
  validateImageDimension(requiredDimension, realDimension) {
    // 无要求自然通过
    if (!requiredDimension || (!requiredDimension.width && !requiredDimension.height && !requiredDimension.maxWidth && !requiredDimension.maxHeight)) {
      return {
        result: true,
        message: '',
      };
    }

    // 指定宽高
    // 如果存在指定宽高且其中一项不符合要求时，视为不正确
    if (requiredDimension.width && requiredDimension.height
      && (!realDimension || realDimension.width !== requiredDimension.width || realDimension.height !== requiredDimension.height)
    ) {
      return {
        result: false,
        message: '尺寸为 ' + requiredDimension.width + 'x' + requiredDimension.height,
      };
    }

    // 限制宽高
    // 如果存在限制宽高且其中一项不符合要求时，视为不正确
    if (requiredDimension.maxWidth && requiredDimension.maxHeight
      && (!realDimension || realDimension.width > requiredDimension.maxWidth || realDimension.height > requiredDimension.maxHeight)
    ) {
      return {
        result: false,
        message: '尺寸不超过 ' + requiredDimension.maxWidth + 'x' + requiredDimension.maxHeight,
      };
    }

    // 其他不符合的情况
    const sections = [];

    // 指定宽度
    if (requiredDimension.width
      && (!realDimension || realDimension.width !== requiredDimension.width)
    ) {
      sections.push('宽度为 ' + requiredDimension.width + 'px');

      // 限制宽度
    } else if (requiredDimension.maxWidth
      && (!realDimension || realDimension.width > requiredDimension.maxWidth)
    ) {
      sections.push('宽度不超过 ' + requiredDimension.maxWidth + 'px');
    }

    // 指定高度
    if (requiredDimension.height
      && (!realDimension || realDimension.height !== requiredDimension.height)
    ) {
      sections.push('高度为 ' + requiredDimension.height + 'px');

      // 限制高度
    } else if (requiredDimension.maxHeight
      && (!realDimension || realDimension.height > requiredDimension.maxHeight)
    ) {
      sections.push('高度不超过 ' + requiredDimension.maxHeight + 'px');
    }

    // 如果有不符存在，拼装信息并返回结果
    if (sections.length > 0) {
      return {
        result: false,
        message: sections.join('，'),
      };
    }

    // 没有发生错误，视为满足条件
    return {
      result: true,
      message: '',
    };
  }

  /**
   * 验证图片容量
   * @param  {JSON} dimension 图片尺寸
   * @param  {int} size       图片容量
   * @return {JSON}           验证结果和返回信息
   */
  validateImageSize(dimension, size) {
    // 没有图片尺寸和容量因为无法验证认为是系统异常
    if (!dimension || !dimension.width || !dimension.height || !size) {
      return {
        result: false,
        message: '系统异常',
      };
    }

    const ratio = 0.4; // 容量对尺寸数值上的比例
    const limitSize = dimension.width * dimension.height * ratio;
    if (size > limitSize) {
      return {
        result: false,
        message: '不超过 ' + Number((limitSize / 1024).toFixed(2)) + 'KB',
      };
    }

    return {
      result: true,
      message: '',
    };
  }
  /**
   * 验证图片是否超过 schema 中规定大小
   *
   * @param {any} file 待验证的文件
   * @param {any} schema 待校验的 schema
   *
   * @memberOf Validator
   * @return {JSON}           验证结果和返回信息
   */
  validateImageMaxSize(file, schema) {
    if (!schema.meta || !schema.meta.maxSize) {
      return {
        result: true,
        message: '',
      };
    }
    const maxSize = schema.meta.maxSize;

    if (isNaN(maxSize)) {
      return {
        result: true,
        message: '',
      };
    }

    if (file.size < maxSize * 1024) {
      return {
        result: true,
        message: '',
      };
    }

    return {
      result: false,
      message: `图片最大为 ${maxSize}KB`,
    };

  }
}

// 单例
module.exports = new Validator();
