import 'should';
import sinon from 'sinon';
import React from 'react';
import schemaUtil from 'schema-util';
import Enzyme, { mount } from 'enzyme';
import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import addKey from '../lib/util/addKey';
import * as locale from '../lib/i18n/zh_cn';
import * as enUS from '../lib/i18n/en';
import FieldArray from '../lib/fields/array';

const mockEvt = {
  stopPropagation() {},
  preventDefault() {},
};

const sandbox = sinon.sandbox.create();

describe('fields/array', () => {
  beforeEach(() => {
    sandbox.restore();
  });

  it('i18n render ok', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, locale: enUS },
    };
    const field = mount(<FieldArray {...props} />);
    expect(field.find('sortableList').length).toEqual(1);
    expect(field.find('ExcelButton').length).toEqual(0);
  });

  it('support excelBtn', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, excelImport: true, locale },
    };
    const field = mount(<FieldArray {...props} />);
    expect(field.find('ExcelButton').length).toEqual(1);
  });

  it('event add and delete item', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    let e;
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: evt => {
        e = evt;
      },
      config: { fieldMap: {}, locale },
    };
    const field = mount(<FieldArray {...props} />);
    field.find('.add-btn a').simulate('click');

    expect(e.value.length).toEqual(data.length + 1);

    // call deleteItem
    field.find('sortableElement(ArrayItem)').at(0).props()
      .deleteItem(mockEvt);
    expect(e.value.length).toEqual(data.length - 1);
  });

  it('event add from zero', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = [];
    let e;
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: evt => {
        e = evt;
      },
      config: { fieldMap: {}, locale },
    };
    const field = mount(<FieldArray {...props} />);
    field.find('.add-btn a').simulate('click');

    expect(e.value.length).toEqual(data.length + 1);
  });

  it('too much items', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    for (let i = 0; i < 510; i++) {
      data.push(Object.assign({}, data[0]));
    }
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, locale },
    };
    const field = mount(<FieldArray {...props} />);
    field.find('sortableElement(ArrayItem)').length.should.below(data.length);
  });

  it('minLength support', function() {
    const schema = schemaUtil.schema('Array(foo) [ minLength: 2 ]{ a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, locale },
    };
    const field = mount(<FieldArray {...props} />);
    field.find('ArrayItem').at(0).props().should.have.property('isHideDelete', true);
  });

  it('toggleItem', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Image }');
    const data = addKey(schemaUtil.mock(schema));
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, locale, validate: () => {} },
    };
    const field = mount(<FieldArray {...props} />);
    const itemProps = field.find('ArrayItem').at(0).props();
    itemProps.toggleItem(0)(mockEvt);
    field.state().should.eql({ actives: { 0: true } });
    // console.log(field.debug());
  });

  it('toggleItem on object', function() {
    const schema = schemaUtil.schema('Array(foo) { a(a), b(b): Object { c(c) }}');
    const data = addKey(schemaUtil.mock(schema));
    let e;
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: ret => { e = ret; },
      config: { fieldMap: {}, locale },
    };
    const field = mount(<FieldArray {...props} />);
    const itemProps = field.find('ArrayItem').at(0).props();
    itemProps.toggleItem(0)(mockEvt);
    field.state().should.not.eql({ actives: { 0: true } });
    e.should.eql({ selected: true, fields: [ 'bar', 0 ] });
    // console.log(field.debug());
  });
});
