import 'should';
import sinon from 'sinon';
import schemaUtil from 'schema-util';

import { read, write } from '../lib/util/csv';

const sandbox = sinon.sandbox.create();

describe('util/csv', function() {

  afterEach(() => {
    sandbox.restore();
  });

  describe('read', function() {
    beforeEach(function() {
      global.XLSX = {
        read: () => ({ SheetNames: [ 'a' ], Sheets: [] }),
        utils: {
          sheet_to_json: (table, config) => {
            if (!config.header) {
              return [{ a1: 1, b1: 2, c1: 'TRUE' }, { a1: 1, b1: '2', c1: 'FALSE' }];
            }
            return [
              [ 'a1', 'b1', 'c1' ],
              [ 1, 2, 'TRUE' ],
              [ 1, '2', 'FALSE' ],
            ];

          },
        },
      };
      FileReader.prototype.readAsBinaryString = function() {
        this.onload({ target: 'a' });
      };
    });

    it('normal', function(done) {
      read({}, 'a').then(ret => {
        ret.should.eql([[ 'a1', 'b1', 'c1' ], [ 1, 2, true ], [ 1, '2', false ]]);
        done();
      }).catch(e => {
        done(e);
      });
    });

    it('multi schema', function(done) {
      global.XLSX.read = () => ({ SheetNames: [ 'b', 'a' ], Sheets: [] });
      read({}, 'a').then(ret => {
        ret.should.eql([[ 'a1', 'b1', 'c1' ], [ 1, 2, true ], [ 1, '2', false ]]);
        done();
      }).catch(done);
    });
  });

  describe('write', function() {
    const schema = schemaUtil.schema(`Array(a) {
      a(a1): Object { d(d) },
      b(b1): Boolean,
      c(c1),
    }`);
    const data = schemaUtil.mock(schema);
    beforeEach(function() {
      global.Workbook = function Workbook() {
        this.name = 'Workbook';
      };
      global.XLSX = {
        write: o => JSON.stringify(o),
        utils: {
          encode_cell: o => '' + o.c + o.r,
          encode_range: o => o,
        },
      };
    });

    it('normal', function(done) {
      global.saveAs = a => {
        const file = new FileReader();
        file.onload = ret => {
          const data = JSON.parse(ret.target.result);
          data.should.have.property('SheetNames', [ 'foo' ]);
          done();
        };
        file.readAsText(a);
      };
      write(schema, data, 'foo');
    });

    const schemaL = schemaUtil.schema(`Lottery(a) {
      a(c1),
      success(s): Array {
        code1(111),
      },
      error(s): Array {
        code2(222),
      },
    }`);
    const dataL = schemaUtil.mock(schemaL);

    it('lottery', function(done) {
      global.saveAs = a => {
        const file = new FileReader();
        file.onload = ret => {
          const data = JSON.parse(ret.target.result);
          data.Sheets.foo['00'].should.eql({ v: 'success', t: 's' });
          done();
        };
        file.readAsText(a);
      };
      write(schemaL, dataL, 'foo');
    });
  });

});
