import 'should';
import sinon from 'sinon';
import React from 'react';
import Enzyme, { mount } from 'enzyme';
import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import * as locale from '../lib/i18n/zh_cn';
import FileField from '../lib/fields/file';

const sandbox = sinon.sandbox.create();

describe('fields/file', () => {
  beforeEach(() => {
    sandbox.restore();
  });

  it('normal', () => {
    const props = {
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: false,
      },
      onChange: target => {
        target.value.should.eql('http://img_url');
      },
    };
    const field = mount(<FileField {...props} />);

    field.find('Upload').length.should.eql(2);
    field.find('Upload').at(0).props().action.should.eql('/api/upload');
  });

  it('不支持的文件类型', () => {
    const props = {
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: false,
      },
      onChange: target => {
        target.value.should.eql('http://img_url');
      },
    };
    const field = mount(<FileField {...props} />);

    field.find('Upload').at(0).props()
      .beforeUpload({
        type: 'jpg',
      });

    field.state().errorMsg.should.eql('不支持该类型的文件');
  });

  it('read file error.', () => {
    const props = {
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: false,
      },
      onChange: target => {
        target.value.should.eql('file content');
      },
    };
    const field = mount(<FileField {...props} />);

    sandbox.stub(global.FileReader.prototype, 'readAsText', function() {
      this.onerror('custom error.');
    });

    field.find('Upload').at(0).props()
      .beforeUpload({
        name: '1.html',
        type: 'text/html',
      });

    field.state().fileName.should.eql('已选择 1.html');
    field.state().errorMsg.should.eql('文件读取错误 custom error.');
  });

  it('process', () => {
    const props = {
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: false,
      },
      onChange: target => {
        target.value.should.eql('file content.');
      },
    };
    const field = mount(<FileField {...props} />);

    sandbox.stub(global.FileReader.prototype, 'readAsText', function() {
      this.onloadend({ target: { readyState: global.FileReader.DONE, result: 'file content.' } });
    });
    sandbox.stub(global, 'setTimeout', function(cb) {
      cb();
    });

    const uploadProps = field.find('Upload').at(0).props();

    field.find('Upload').at(0).props()
      .beforeUpload({
        type: 'text/html',
      });

    uploadProps.onChange({
      file: { status: 'error', response: 'custom error.' },
    });
    field.state().errorMsg.should.eql('custom error.');

    uploadProps.onChange({
      file: { status: 'new' },
    });

    uploadProps.onChange({
      file: { status: 'done', response: { url: 'file content.' } },
    });
    field.state().errorMsg.should.eql('');
  });

});
