import 'should';
import React from 'react';
import schemaUtil from 'schema-util';
import Enzyme, { shallow, mount } from 'enzyme';
import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import Editor from '../lib/index';
import ObjectField from '../lib/fields/object';
import * as locale from '../lib/i18n/zh_cn';
import { DevTool } from '../lib/util';

describe('render', () => {
  beforeEach(() => {
    global.requestAnimationFrame = function(callback) {
      setTimeout(callback, 0);
    };
  });

  it('normal', () => {
    const schema = schemaUtil.schema('Object(foo) { a(a): URL, b(b) }');
    const props = {
      data: schemaUtil.mock(schema),
      schema,
      onChange: () => {},
    };
    const editor = shallow(<Editor {...props}/>);
    expect(editor.find(ObjectField).length).toEqual(1);
  });

  it('object nest', function() {
    const schema = schemaUtil.schema('Object(foo) { a(a): URL, b(b): Object { c(c), d(d) } }');
    const props = {
      data: schemaUtil.mock(schema),
      schema,
      onChange: () => {},
    };
    const editor = shallow(<Editor {...props}/>);
    expect(editor.find('BreadcrumbItem').html().includes('foo')).toEqual(true);

    editor.find(ObjectField).props().onChange({ selected: true, fields: [ 'b' ] });

    expect(editor.find('BreadcrumbItem').length).toEqual(2);
    const nextProps = editor.find(ObjectField).props();
    expect(nextProps.data).toEqual({ c: 'c', d: 'd' });
  });

  it('object nest array', () => {
    const schema = schemaUtil.schema(`Array(foo) {
      b(b): Array { c(c), d(d), e(e): Object { f(f) } },
      a(a): URL
    }`);
    const props = {
      data: schemaUtil.mock(schema),
      schema,
      onChange: () => {},
    };
    const editor = shallow(<Editor {...props}/>);
    const arrayField = editor.find('SortArray');
    expect(arrayField.length).toEqual(1);
    const onChange = arrayField.props().onChange;
    // select number
    onChange({ selected: true, fields: [ 0 ] });

    expect(editor.find('BreadcrumbItem').length).toEqual(2);
    expect(editor.find('BreadcrumbItem').at(0).children()
      .text()
      .trim()).toEqual('foo');

    // select next b
    onChange({ selected: true, fields: [ 'b' ] });
    expect(editor.find('BreadcrumbItem').length).toEqual(3);
    expect(editor.find('BreadcrumbItem').at(2).children()
      .text()
      .trim()).toEqual('b');

    editor.find('BreadcrumbItem').at(1).simulate('click');
    expect(editor.find('BreadcrumbItem').length).toEqual(2);
    expect(editor.state().selected).toEqual([ 0 ]);
  });

  describe('onChange', function() {
    it('仅字段值修改', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), d(d), e(e): Object { f(f) } },
        a(a): URL
      }`);
      const props = {
        data: schemaUtil.mock(schema),
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 'a.com',
        fields: [ 'a' ],
      });
      editor.state().data.should.have.property('a', 'a.com');
      onChange({
        value: 'bar',
        fields: [ 'b', 1, 'e', 'f' ],
      });
      editor.state().data.b[1].e.should.eql({ f: 'bar' });
    });

    it('字段新增/减少', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e): Object { f(f) }},
      }`);
      const props = {
        data: { b: [{ c: 'c', d: 'd' }, { c: 'c', d: 'd' }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 'new',
        fields: [ 'b', 1, 'c' ],
      });
      editor.state().data.b[1].c.should.eql('new');
      // console.log(1, editor.state().data);
      // 注意：老的 d 字段数据还存在
      // { b: [ { c: 'c', d: 'd', __key: 1 }, { c: 'new', d: 'd', __key: 2 } ] }
      onChange({
        value: 'f',
        fields: [ 'b', 1, 'e', 'f' ],
      });
      editor.state().data.b[1].e.f.should.eql('f');
    });

    it('字段类型变化: 简单类型 -> 简单类型', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e): Number },
      }`);
      const props = {
        data: { b: [{ c: 'c', e: 'e' }, { c: 'c', e: 'e' }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 123,
        fields: [ 'b', 1, 'e' ],
      });
      editor.state().data.b[1].e.should.eql(123);
    });

    it('字段类型变化: 对象 -> 简单类型', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e) },
      }`);
      const props = {
        data: { b: [{ c: 'c', e: { f: 'f' } }, { c: 'c', e: { f: 'f' } }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 'new',
        fields: [ 'b', 1, 'e' ],
      });
      editor.state().data.b[1].e.should.eql('new');
    });

    it('字段类型变化: 数组 -> 简单类型', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e) },
      }`);
      const props = {
        data: { b: [{ c: 'c', e: [{ f: 'f' }] }, { c: 'c', e: [{ f: 'f' }] }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 'new',
        fields: [ 'b', 1, 'e' ],
      });
      editor.state().data.b[1].e.should.eql('new');
    });

    it('字段类型变化: 简单类型 -> 数组', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e): Array { f(f) }},
      }`);

      const props = {
        data: { b: [{ c: 'c', e: 'e' }, { c: 'c', e: 'e' }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: [{ f: 'f' }],
        fields: [ 'b', 1, 'e' ],
      });
      editor.state().data.b[1].e.should.eql([{ f: 'f' }]);
    });

    it('字段类型变化: 简单类型 -> 对象', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e): Object { f(f): Object { g(g) } }},
      }`);
      const props = {
        data: { b: [{ c: 'c', e: 'e' }, { c: 'c', e: 'e' }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: 'new',
        fields: [ 'b', 1, 'e', 'f', 'g' ],
      });
      editor.state().data.b[1].e.f.g.should.eql('new');
    });

    it('数组项增加或减少', function() {
      const schema = schemaUtil.schema(`Object(foo) {
        b(b): Array { c(c), e(e)},
      }`);
      const props = {
        data: { b: [{ c: 'c', e: 'e' }, { c: 'c', e: 'e' }] },
        schema,
        onChange: () => {
        },
      };
      const editor = shallow(<Editor {...props}/>);
      const onChange = editor.find(ObjectField).props().onChange;
      onChange({
        value: [{ c: 'c', e: 'e' }, { c: 'c', e: 'e' }, { c: 'new', e: 'new' }],
        fields: [ 'b' ],
        type: 'array-add',
      });
      editor.state().data.b[2].should.eql({ c: 'new', e: 'new' });
      onChange({
        value: [{ c: 'c', e: 'e' }],
        fields: [ 'b' ],
        type: 'array-delete',
      });
      editor.state().data.b.length.should.eql(1);
    });

  });

  it('validate', function() {
    const schema = schemaUtil.schema(`Object(foo) {
      b(b): Array { c(c), d(d), e(e): Object { f(f) } },
      a(a): URL
    }`);
    const props = {
      data: schemaUtil.mock(schema),
      schema,
      onChange: () => {},
      onError: e => { e; },
    };
    const editor = shallow(<Editor {...props}/>);
    const validate = editor.find(ObjectField).props().config.validate;
    const ret = validate('a. com', {}, 'url', locale);
    ret.should.eql('URL 格式不正确');

    validate('a.png', {}, 'image', locale).should.eql('图片格式不正确');
  });

  it('import and export support', function() {
    const schema = schemaUtil.schema('Object(foo) { a(a): URL, b(b) }');
    let data = { a: '12', b: 'b' };
    const props = {
      data,
      schema,
      onChange: ret => {
        data = ret;
      },
    };
    const editor = shallow(<Editor {...props}/>);
    const uid = editor.getElement().props['data-schema-uid'];
    const ret = DevTool.exportData(uid);
    ret.should.eql(data);

    const ret1 = DevTool.exportData();
    ret1.should.eql(data);

    DevTool.importData({ a: 'foo', b: 'bar' }, uid);
    editor.find('FieldObject').props().data.should.eql({ a: 'foo', b: 'bar' });
  });

  it('enum should show label, when $root source is outline.', function() {
    const schema = schemaUtil.schema(`Object(config) {
      option(option): Array {
        label(n),
        value(v)
      },
      list(list): Array {
        select(select): Enum [ source: "$root.option"]
      }
    }`);

    const data = {
      list: [
        { select: 'value_1' },
      ],
      option: [
        { __key: 1, value: 'value_1', label: 'label_1' },
        { __key: 2, value: 'value_2', label: 'label_2' },
      ],
    };

    const props = {
      data,
      schema,
      onChange: () => { },
      config: {
        fieldMap: {},
        validate: () => null,
        locale,
        data,
        paths: [],
      },
    };
    const field = mount(<Editor {...props} />);
    field.find('.ui-object-folder').last().simulate('click');
    field.find('.row-title').text().should.eql('label_1');
  });

});
