import React from 'react';
import Enzyme, { shallow, mount } from 'enzyme';
import schemaUtil from 'schema-util';
import 'should';
import { Select } from 'antd';
import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import ObjectField from '../lib/fields/object';
import * as locale from '../lib/i18n/zh_cn';

describe('fields/object', () => {
  it('render simple object ok', () => {
    const schema = schemaUtil.schema(`Object(foo) {
      a(a),
      b(b): Image
    }`);
    const data = schemaUtil.mock(schema);
    const props = {
      data,
      schema,
      field: 'bar',
      onChange: () => {},
      config: { fieldMap: {}, locale },
    };
    const field = shallow(<ObjectField {...props} />);
    // console.log(field.debug());
    expect(field.find('.ui-title').text()).toEqual('foo');
    expect(field.find('Textfield').length).toEqual(1);
    expect(field.find('Uploader').length).toEqual(1);
  });

  it('do not have .ui-title', () => {
    const schema = schemaUtil.schema('Object(foo) { a(a), b(b): Image }');
    const data = schemaUtil.mock(schema);
    const props = {
      data,
      schema,
      onChange: () => {},
      config: { fieldMap: {}, locale },
    };
    const field = shallow(<ObjectField {...props} />);
    expect(field.find('.ui-title').length).toEqual(0);
  });

  it('object nest object', () => {
    let calledOnce = false;
    const schema = schemaUtil.schema('Object(foo) { a(a): Object { b(b): Image } }');
    const data = schemaUtil.mock(schema);
    const props = {
      data,
      schema,
      onChange: () => { calledOnce = true; },
      config: { fieldMap: {}, locale },
    };
    const field = shallow(<ObjectField {...props} />);
    // console.log(field.debug());
    expect(field.find('.ui-object-folder-editor').length).toEqual(1);

    field.find('.ui-object-folder').simulate('click');
    expect(calledOnce).toEqual(true);
  });

  it('if support', () => {
    const schema = schemaUtil.schema(`Object(foo) {
      a(a): Boolean,
      c(c): String [if: "$.a"],
      d(d): Color [if: "!$.a"],
      e(e): Number [if: "!a"],
    }`);
    const data = schemaUtil.mock(schema);
    const props = {
      data,
      schema,
      onChange: () => {},
      config: {
        fieldMap: {},
        locale,
      },
    };
    const field = shallow(<ObjectField {...props} />);
    // console.log(field.debug());
    field.find('Textfield').length.should.equal(0);
    field.find('.field-color').length.should.equal(1);

    data.a = true;
    field.setProps({ data });
    field.update();
    field.find('Textfield').length.should.equal(1);
    // console.log(field.debug());
  });

  it('render all field', () => {
    const schema = schemaUtil.schema(`Object(foo) {
      title(标题): String [ minLength: 4, maxLength: 10 ],
      image(图片): Image,
      url(URL): URL,
      bool(布尔): Boolean,
      num(数字): Number [step: 0.1, max: 20, min: 15],
      text(多行文本): Text,
      rtext(富文本): RichText,
      date(日期): Date,
      color(颜色): Color,
      enum(枚举): Enum {
        a(值A),
        b(值B)
      },
      file(file): File,
    }`);
    const data = schemaUtil.mock(schema);
    const props = {
      data,
      schema,
      onChange: () => { },
      config: { fieldMap: {}, validate: () => null, locale },
    };
    const field = mount(<ObjectField {...props} />);
    expect(field.find('.ui-object > .field-string').length).toEqual(1);
    expect(field.find('.field-image').length).toEqual(1);
    expect(field.find('.field-url').length).toEqual(1);
    expect(field.find('Switcher').length).toEqual(1);
    expect(field.find('NumberField').length).toEqual(1);
    expect(field.find('.field-text').length).toEqual(1);
    expect(field.find('.field-richtext').length).toEqual(1);
    expect(field.find('DateField').length).toEqual(1);
    expect(field.find('.ui-object > .field-color').length).toEqual(1);
    expect(field.find('.field-enum').length).toEqual(1);
    expect(field.find('.item-file').length).toEqual(1);
    // console.log(field.debug());
  });

  it('RichText XSS Detect', () => {
    const schema = schemaUtil.schema(`Object(foo) {
      rtext(富文本): RichText,
    }`);
    const data = {
      rtext: '<script>0此处将被删掉</script><a href="alipays://plat/&url=https%3A%2">1自定义属性</a><a href="//alipay.com" target="_blank">2target属性</a><img onload=alert(1) src="" /><A href="javascript:openwindow()">3过滤伪协议</A><a href="java&Tab;scri&Tab;pt:alert(1)">4</a>',
    };
    const targetData = '<div class="rich-text">&lt;script&gt;0此处将被删掉&lt;/script&gt;<a href="">1自定义属性</a><a href="//alipay.com" target="_blank">2target属性</a><img src=""><a href="">3过滤伪协议</a><a href="">4</a></div>';
    const props = {
      data,
      schema,
      onChange: () => { },
      config: { fieldMap: {}, validate: () => null, locale },
    };
    const field = mount(<ObjectField {...props} />);
    expect(field.find('.field-richtext').find('.rich-text').html()).toEqual(targetData);
  });

  describe('RichText XSS white list', () => {
    it('support style attr', () => {
      const schema = schemaUtil.schema(`Object(foo) {
        context(富文本): RichText,
      }`);
      const data = {
        context: '<div style="font-size:12px;background:red;">背景色</div><b style="color:red;">b</b><h1 style="background:red;">test</h1><img src="http://src.jpg" style="width:100px;" /><p style="text-decoration:underline;">p</p><span style="color:red;">文字颜色</span>',
      };
      const targetData = '<div class="rich-text"><div style="font-size:12px;">背景色</div><b style="color:red;">b</b><h1 style="">test</h1><img src="http://src.jpg" style="width:100px;"><p style="text-decoration:underline;">p</p><span style="color:red;">文字颜色</span></div>';
      const props = {
        data,
        schema,
        onChange: () => { },
        config: { fieldMap: {}, validate: () => null, locale },
      };
      const field = mount(<ObjectField {...props} />);
      expect(field.find('.field-richtext').find('.rich-text').html()).toEqual(targetData);
    });

    it('style whitelist', () => {
      const schema = schemaUtil.schema(`Object(foo) {
        context(富文本): RichText,
      }`);
      const data = {
        context: '<span style="width:100px;height:100px;color:red;background:red;font-size:12px;text-align:right;position:absolute;top:0;left:0;">文字颜色</span>',
      };
      const targetData = '<div class="rich-text"><span style="width:100px; height:100px; color:red; font-size:12px; text-align:right;">文字颜色</span></div>';
      const props = {
        data,
        schema,
        onChange: () => { },
        config: { fieldMap: {}, validate: () => null, locale },
      };
      const field = mount(<ObjectField {...props} />);
      expect(field.find('.field-richtext').find('.rich-text').html()).toEqual(targetData);
    });
  });

  it('enum source support', () => {
    const schema = schemaUtil.schema(`Object(enums) {
      enum(锚点选择): Enum [ source: "urls" ] {
        a(a)
      },
      urls(urls): Array {
        value(链接)
        label(名字)
      }
    }`);
    const data = {
      enum: 'a',
      urls: [
        { __key: 1, value: '1', label: 'label1' },
        { __key: 2, value: '2', label: 'label2' },
      ],
    };

    const props = {
      data,
      schema,
      onChange: () => { },
      config: { fieldMap: {}, validate: () => null, locale },
    };
    const field = mount(<ObjectField {...props} />);
    field.find(Select).props().children.length.should.equal(2);
  });

  it('enum source support $root.', () => {
    const schema = schemaUtil.schema(`Object(enums) {
      enum(锚点选择): Enum [ source: "$root.urls" ]
    }`);
    const data = {
      enum: 'a',
    };

    const rootData = {
      a: { enum: 'foo' },
      urls: [
        { __key: 1, value: '1', label: 'label1' },
        { __key: 2, value: '2', label: 'label2' },
      ],
    };

    const props = {
      data,
      schema,
      onChange: () => { },
      config: {
        fieldMap: {},
        validate: () => null,
        locale,
        rootData,
        paths: [ 'a' ],
      },
    };
    const field = mount(<ObjectField {...props} />);
    field.find(Select).props().children.length.should.equal(2);
  });

  it('enum source support $root. on array', () => {
    const schema = schemaUtil.schema(`Object(enums) {
      enum(锚点选择): Enum [ source: "$root.urls" ]
    }`);
    const data = {
      enum: 'a',
    };

    const rootData = [{
      a: { enum: 'foo' },
      urls: [
        { __key: 1, value: '1', label: 'label1' },
        { __key: 2, value: '2', label: 'label2' },
      ],
    }, { urls: [] }];

    const props = {
      data,
      schema,
      onChange: () => { },
      config: {
        fieldMap: {},
        validate: () => null,
        locale,
        rootData,
        paths: [ 0, 'a' ],
      },
    };
    const field = mount(<ObjectField {...props} />);
    field.find(Select).props().children.length.should.equal(2);
  });

  it('getGroupsRule', () => {
    const schema = schemaUtil.schema(`Object(enums) {
      enum(锚点选择): Enum [ source: "$root.urls" ]
    }`);
    const data = {
      enum: 'a',
    };

    const rootData = [{
      a: { enum: 'foo' },
      urls: [
        { __key: 1, value: '1', label: 'label1' },
        { __key: 2, value: '2', label: 'label2' },
      ],
    }, { urls: [] }];

    const props = {
      data,
      schema,
      onChange: () => { },
      getGroupsRule: data => {
        (data.length > 0).should.eql(true);
      },
      config: {
        fieldMap: {},
        validate: () => null,
        locale,
        rootData,
        paths: [ 0, 'a' ],
      },
    };
    mount(<ObjectField {...props} />);
  });
});
