import 'should';
import sinon from 'sinon';
import React from 'react';
import Enzyme, { mount } from 'enzyme';
import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import * as locale from '../lib/i18n/zh_cn';
import UploaderField from '../lib/fields/uploader';

const sandbox = sinon.sandbox.create();

describe('fields/uploader', () => {
  afterEach(() => {
    sandbox.restore();
  });

  it('imageUploadOnly: true', () => {
    const props = {
      field: '头像',
      data: 'http://url',
      schema: {
        type: 'customType',
        meta: {
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
        },
      },
      config: {
        locale,
        validate: () => { return null; },
        imageUploadOnly: true,
      },
      onChange: () => {
      },
    };
    const field = mount(<UploaderField {...props} />);

    field.find('.item-image-preview').length.should.eql(1);
    field.find('Copy').length.should.eql(1);
  });

  it('meta.maxSize', () => {
    const props = {
      field: '头像',
      data: 'http://url',
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
        },
      },
      config: {
        locale,
        validate: () => { return null; },
        imageUploadOnly: true,
      },
      onChange: () => {
      },
    };
    const field = mount(<UploaderField {...props} />);
    field.find('Upload').at(0).props()
      .beforeUpload({
        size: 100 * 1024,
      });

    field.state().errorMsg.should.eql('图片最大为 10KB');
  });

  it('config.uploadImgSize', () => {
    const props = {
      field: '头像',
      data: 'http://url',
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 1024,
        validate: () => { return null; },
        imageUploadOnly: true,
      },
      onChange: () => {
      },
    };
    const field = mount(<UploaderField {...props} />);
    field.find('Upload').at(0).props()
      .beforeUpload({
        size: 2048,
        type: 'image.gif',
      });

    field.state().errorMsg.should.eql('图片大小超过 1KB');
  });

  it('process', () => {
    const imgUrl = [];
    const props = {
      field: '头像',
      data: 'http://url',
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: true,
      },
      onChange: target => {
        imgUrl.push(target);
      },
    };
    const field = mount(<UploaderField {...props} />);

    sandbox.stub(global.FileReader.prototype, 'readAsDataURL', function() {
      this.onloadend({ target: { readyState: global.FileReader.DONE, result: 'http://' } });
    });
    sandbox.stub(global, 'setTimeout', function(cb) {
      cb();
    });

    const uploadProps = field.find('Upload').at(0).props();
    uploadProps.beforeUpload({
      size: 2048,
      type: 'image.gif',
    });
    uploadProps.onChange({
      file: { status: 'error', response: 'custom error.' },
    });

    field.state().errorMsg.should.eql('custom error.');
    uploadProps.onChange({
      file: { status: 'customStatus' },
    });
    field.state().errorMsg.should.eql('custom error.');

    uploadProps.onChange({
      file: { status: 'done', response: { url: 'http://new_url' } },
    });
    field.state().errorMsg.should.eql('');
    field.state().success.should.eql(false);

    // 删除图片
    field.find('a').length.should.eql(0);
  });

  it('changeImage', () => {
    const props = {
      field: '头像',
      data: 'http://url',
      schema: {
        type: 'customType',
        meta: {
          maxSize: 10,
          width: 100,
          height: 100,
          maxWidth: 300,
          maxHeight: 300,
          setDefaultColorTo: 'a',
        },
      },
      config: {
        locale,
        uploadImgSize: 4058,
        validate: () => { return null; },
        imageUploadOnly: false,
      },
      onChange: target => {
        target.value.should.eql('http://img_url');
      },
    };
    const field = mount(<UploaderField {...props} />);

    field.find('Input').at(0).props()
      .onChange({ target: { value: 'http://img_url' } });

    field.state().errorMsg.should.eql('');
  });
});
