require('should');
import React from 'react';
import schemaUtil from 'schema-util';
import Enzyme, { mount } from 'enzyme';

import Adapter from 'enzyme-adapter-react-16';


Enzyme.configure({ adapter: new Adapter() });

import Copy from '../lib/util/copy';
import addKey from '../lib/util/addKey';
import evaluate from '../lib/util/evaluate';
import getArrayTitle from '../lib/util/getArrayTitle';

describe('util/getArrayTitle', () => {
  it('from title or name', function() {
    const title = getArrayTitle({ a: 1, name: 2 }, schemaUtil.schema(`Array(a) {
      a(xxx): number,
      name(xx)
    }`));
    title.should.equal(2);

    const title1 = getArrayTitle({ a: 1, title: 2 }, schemaUtil.schema(`Array(a) {
      a(xxx): number,
      title(xx)
    }`));
    title1.should.equal(2);
  });

  it('object from __key', function() {
    const title = getArrayTitle({
      a: { b: 1 },
      name: [],
      __key: 1,
    }, schemaUtil.schema(`Array(descA) {
      a(xxx): Object { b(b) },
      name(xx): Array { c(c) }
    }`));
    title.should.equal('descA1');
  });

  it('get enum label', function() {
    const title = getArrayTitle({ a: '2', name1: 2 }, schemaUtil.schema(`Array(a) {
      a(xxx):Enum {
        1(val1),
        2(val2)
      },
      name1(xx)
    }`));
    title.should.equal('val2');
  });

  it('get box label', function() {
    const data = { a: { left: 16, top: 73.25, width: 69, height: 12 }, name1: 2, __key: 1 };
    const title = getArrayTitle(data, schemaUtil.schema(`Array(a) {
      a(xxx):Box,
      name1(xx)
    }`));
    title.should.equal('a1');
  });

  it('define showKey', function() {
    const schema = schemaUtil.schema(`Array(a) [
        showKey: "a"
      ] {
      a(xxx): number,
      name(xx)
    }`);
    const title = getArrayTitle({ a: 1, name: 2 }, schema);
    title.should.equal(1);
  });

  it('add key', function() {
    const data = [{ __key: 1 }, { __key: 3 }, {}, {}];
    addKey(data);
    data.should.eql([{ __key: 1 }, { __key: 3 }, {}, {}]);
  });

  it('error showKey', function() {
    const title = getArrayTitle({ a: 1, name: 2 }, schemaUtil.schema(`Array(a) {
      a(xxx): number,
      name1(xx)
    }`));
    title.should.equal(1);
  });

  describe('evaluate', () => {
    it('basic expression', () => {
      const ret = evaluate(null, '1 * (2 + 3 * (4 - 3))');
      ret.should.equal(5);
      const ret1 = evaluate(null,
        '1 * (2 + 3 * (4 - 3)) > 10 - 2 || 3 * 2 > 9 - 2 * 3');
      ret1.should.equal(true);
      evaluate(null, '(9 - 2) * 3 - 10').should.equal(11);

      [
        '9 / 12 + 12 * 3 - 5',
        '9 / 12 + 12 * (3 - 5)',
        '12 > 13.1',
        '12 < 14',
        '12 <= 14',
        '12 >= 14',
        '12 == 14',
        '12 != 14',
        '9 - 1 > 10 && 3 * 5 > 10',
        '9 - 1 > 10 || 3 * 5 > 10',
      ].forEach(expression => {
        /* eslint-disable-next-line */
        evaluate(null, expression).should.equal(eval(expression));
      });
    });

    it('read var from context', () => {
      evaluate({ a: 10 }, '(9 - 2) * 3 - $.a').should.equal(11);
      evaluate({ a: 10, b: 2 }, '(9 - $.b) * 3 - $.a').should.equal(11);
      evaluate({ a: 10, b: 2 }, '$.a > $.b').should.equal(true);
      evaluate({ a: 10, b: 2 }, '$.a > $.b == false').should.equal(false);
      evaluate({ a: 10, b: 2 }, '$.a > $.b == true').should.equal(true);
      evaluate({ a: 'foo' }, '$.a == \'foo\'').should.equal(true);
      evaluate({ a: 'foo' }, '$.a == "foo" && 1 > 0').should.equal(true);

      evaluate({ a: 'foo' }, '!!$.a').should.equal(true);
      evaluate({ a: 'foo' }, '!$.a').should.equal(false);
      evaluate({ a: 'foo' }, '$.a').should.equal('foo');
      evaluate({ a: 'foo' }, '!($.a > "foa") || 1 > 2').should.equal(false);
      evaluate({ a: 'foo' }, '!($.a > "foa") || 1 < 2').should.equal(true);
    });

    it('string parse', () => {
      evaluate({ a: '1>2' }, '$.a == "1>2"').should.equal(true);
      evaluate({ a: '' }, '$.a == ""').should.equal(true);
      evaluate({ a: '"' }, '$.a == \'"\'').should.equal(true);
      evaluate({ a: '\'a\'' }, '$.a == "\'a\'" && $.a != "a"').should.equal(true);
    });
  });
});

describe('addKey', () => {
  it('data is null', () => {
    const data = addKey(null);
    (!data).should.eql(true);
  });

  it('data is object', () => {
    const data = addKey({ a: 2 });
    data.should.eql({
      a: 2,
    });
  });
});

describe('copy', () => {
  it('normal', () => {
    const props = {
      text: 'http://url',
      className: 'custom-copy',
      description: '复制标题',
    };
    const field = mount(<Copy {...props}><span>复制 url</span></Copy>);

    field.find('Trigger').length.should.eql(1);
    field.find('Tooltip').at(0).props().title.should.eql(props.description);
    field.find('CopyToClipboard').at(0).props().text.should.eql(props.text);
    field.find('span').at(0).props().className.should.eql('common-copy custom-copy');

    const copyProps = field.find('CopyToClipboard').at(0).props();
    copyProps.onCopy();

    field.state().should.eql({ copied: true });
  });
});
